--=======================================================================
--miniSIPServer call-park service script
--
--Copyright (C) MYVOIPAPP,Inc. All Rights Reserved.
--
--Author: Gilson
--
--History:
--2014-02-27 Migrated from MSS core.
--2016-09-14 Migrated from python script.
--=======================================================================
require "services"
require "utils.data_string"
require "utils.dba"

--service states
local ECALLPARK_IDLE = 0
local ECALLPARK_WAIT_LEG_RELD = 1
local ECALLPARK_WAIT_RETRIEVE = 2

-- Timer values
local CALLPARK_TIMER_WAIT_LEG_RELD = 10
local CALLPARK_TIMER_WAIT_RETRIEVE = 60

-- call park and retrieve
local MIN_CALLPARK_CHANNEL = 1

--====================================
--
-- Cache DB functions
--
--====================================
local function cdbCallParkSet()
    TraceInfo("\t cdbCallParkSet")
    if fsm.channel<=0 then return -1 end

    local sqlStr=string.format("insert into tbl_callpark (channel, callerSRFID, ssfID, scfID) "..
                 "values (%d, %d, %d,%d)", fsm.channel, fsm.callerSrfID, GetSsfFsmID(0), fsm.scpFsmID)
    local result=cdbQuery(sqlStr)
    if result~=MSS_ERR_OK then
        TraceInfo("\t\t Fail to insert data. SQL: "..sqlStr)
        return -2
    end
    return 0                  
end


local function cdbCallParkDel()
    TraceInfo("\t cdbCallParkDel")
    local sqlStr=string.format("delete from tbl_callpark where "..
                 "channel=%d and scfID=%d", fsm.channel, fsm.scpFsmID)
    local result=cdbQuery(sqlStr)
    if result~=MSS_ERR_OK then
        TraceInfo("\t\t Fail to del data. SQL: "..sqlStr)
        return -2
    end
    return 0                  
end


local function cdbCallParkHas()
    local sqlStr=string.format("select count(*) from tbl_callpark where channel=%d", fsm.channel)
    local result=cdbQuery(sqlStr)
    if result~=MSS_ERR_OBJ_FOUND then return MSS_NO end

    result = cdbGetRowIntVal(0)
    if result<=0 then return MSS_NO else return MSS_YES end
end


--====================================
--
-- FSM common functions
--
--====================================
local function initParams()
    fsm.callparkPrefix = ssfGetSysVarCallParkPrefix()
    fsm.channel = 0    
end


local function procException()
    TraceInfo("\t procException")
    SendReleaseCall()
    EndService()
end


local function saveIDPInfo()
    TraceInfo("\t saveIDPInfo")
    if fsm.callerSrfID==MSS_INVALID_OBJ_ID then return -1 end
    if fsm.serviceNbr=="" then return -2 end
    if MSS_FALSE==StrHasPrefix(fsm.serviceNbr, fsm.callparkPrefix) then return -3 end

    local channelStr=StrLstrip(fsm.serviceNbr, fsm.callparkPrefix)
    channelStr = StrRstrip(channelStr, "#")
    fsm.channel = ToInt(channelStr)
    if fsm.channel<MIN_CALLPARK_CHANNEL then return -4 end
    
    TraceInfo("\t\t channel="..fsm.channel)
    return 1
end


local function monitorCall()
    TraceInfo("\t monitorCall")
    SendRRBE(PASSIVE_LEGID, DP_T_DISCONNECT, DP_MON_NOTIFY)
end


local function saveCallParkInfo()
    TraceInfo("\t saveCallParkInfo")
    if cdbCallParkHas()==MSS_YES then
        cdbCallParkDel()
    end
    cdbCallParkSet()
end


local function playMusic()
    TraceInfo("\t playMusic")
    SendCTR(PASSIVE_LEGID)

    local paArg = CreatePA()
    paArg.infoSend_annID_annID = ANN_ID_COMMON_MUSIC
    paArg.infoSend_duration = CALLPARK_TIMER_WAIT_RETRIEVE+10
    SendPA(PASSIVE_LEGID, paArg)
end

--====================================
--
-- FSM state functions
--
--====================================
local function onIdleProc()
    TraceInfo("callPark: onIdleProc")
    initParams()

    local result = saveIDPInfo()
    if result<0 then 
        procException() 
        return
    end

    monitorCall()
    SendDisconnectLeg(CONTROL_LEGID)

    StartTimer(CALLPARK_TIMER_WAIT_LEG_RELD)
    EnterState(ECALLPARK_WAIT_LEG_RELD)
end


local function onWaitLegReldProc(currMsg)
    TraceInfo("onWaitLegReldProc")
    if currMsg~=ESCP_EVT_LEG_RELD then 
        procException()
        return
    end

    StopTimer()
    playMusic()
    saveCallParkInfo()

    StartTimer(CALLPARK_TIMER_WAIT_RETRIEVE)
    EnterState(ECALLPARK_WAIT_RETRIEVE)        
end


local function onWaitRetrieveProc(currMsg)
    TraceInfo("onWaitRetrieveProc")
    if currMsg~=ESCP_EVT_SCF_CALL_RETRIEVE_REQ then
        procException()
        return
    end

    StopTimer()
    SendDFC(PASSIVE_LEGID)
    SendTransClose()
    SendCallRetrievedResp(fsm.orgFsmID)
    EndService()
end


--====================================
--
-- Service main function
--
--====================================
function DestroyProc()
    TraceInfo("DestroyProc")
    if fsm.channel>0 then cdbCallParkDel() end
end


function MainProc(currMsg)
    if fsm.state==ECALLPARK_IDLE then
        onIdleProc()

    elseif fsm.state==ECALLPARK_WAIT_LEG_RELD then
        onWaitLegReldProc(currMsg)

    elseif fsm.state==ECALLPARK_WAIT_RETRIEVE then
        onWaitRetrieveProc(currMsg)

    else
        TraceInfo("unknown state: "..fsm.state)
        procException()
    end
end