--==============================================================================
--miniSipServer Direct Inward System Access service script
--
--Copyright (C) MYVOIPAPP,Inc. All Rights Reserved.
--
--author: Gilson
--
--History:
--2016-08-11 Migrated from Python script
--==============================================================================

require "services"
require "utils.dba"

--service states
local EDISA_STATE_IDLE          = 0
local EDISA_STATE_WAIT_ACCOUNT  = 1
local EDISA_STATE_WAIT_PWD      = 2
local EDISA_STATE_WAIT_DEST_NBR = 3
local EDISA_STATE_WAIT_ANS      = 4

--announcement IDs
local DISA_ANNID_INPUT_ACCOUNT = 0x85080001
local DISA_ANNID_INPUT_PWD     = 0x85080002
local DISA_ANNID_INPUT_DEST    = 0x85080003

--default timer value (seconds)
local DISA_TIMR_WAIT_USER_INPUT       = 100 
local DISA_TIMR_FIRST_DIGIT_TIMEOUT   = 50 
local DISA_TIMR_WAIT_ANS              = 120

--====================================
--
-- database functions
--
--====================================
local function db_get_disa_account()
    TraceInfo("\t db_get_disa_account")

    local sqlStr = string.format("select cDISAAccount from tbl_disa_caller_id where "..
                    "cCallerID='%s'", fsm.callerNbr)
    DbPrepare()
    local result = DbQuery(sqlStr)
    if MSS_ERR_OK~=result or 0==DbGetRecordCnts() then
        DbEnd()
        return -1
    end

    DbFetchRow()
    fsm.disa_account = DbGetRowStrVal(0)
    TraceInfo("\t\t disa account="..fsm.disa_account)
    DbEnd()
    return 1
end

local function db_get_disa_extension()
    TraceInfo("\t db_get_disa_extension")
    local sqlStr=string.format("select cExtension from tbl_disa_account where "..
                 "cAccount='%s'", fsm.disa_account)
    DbPrepare()
    local result = DbQuery(sqlStr)
    if MSS_ERR_OK~=result or 0==DbGetRecordCnts() then
        DbEnd()
        return -1
    end

    DbFetchRow()
    fsm.disa_extension = DbGetRowStrVal(0)
    TraceInfo("\t\t disa extension = "..fsm.disa_extension)
    DbEnd()
    return 1
end

local function db_check_caller_id()
    TraceInfo("\t db_check_caller_id")
    local result = db_get_disa_account()
    if result<0 then return 0 end

    result = db_get_disa_extension()
    if result<=0 then return -1 end --this account is not bind to an extension
    
    return 1
end

local function db_check_disa_account_info()
    TraceInfo("\t db_check_disa_account_info")
    local sqlStr=string.format("select cExtension from tbl_disa_account where "..
                "cAccount='%s' and cPassword ='%s'", fsm.disa_account, fsm.disa_password)
    DbPrepare()
    local result = DbQuery(sqlStr)
    if MSS_ERR_OK~=result or 0==DbGetRecordCnts() then 
        DbEnd()
        return -1
    end

    DbFetchRow()
    fsm.disa_extension = DbGetRowStrVal(0)
    TraceInfo("\t\t disa extension = "..fsm.disa_extension)
    DbEnd()
    return 1
end

--====================================
--
-- IVR functions
--
--====================================
local function askCallerInputPassword()
    TraceInfo("\t askCallerInputPassword")
    SendPNC(CALLER_LEGID, DISA_ANNID_INPUT_PWD, DISA_TIMR_FIRST_DIGIT_TIMEOUT, 5)
    StartTimer(DISA_TIMR_WAIT_USER_INPUT)
    EnterState(EDISA_STATE_WAIT_PWD)
end

local function askCallerInputDestNbr()
    TraceInfo("\t askCallerInputDestNbr")
    if fsm.ctrInd==MSS_NO then 
        fsm.ctrInd = MSS_YES
        SendCTR(CALLER_LEGID)
    end

    SendPNC(CALLER_LEGID, DISA_ANNID_INPUT_DEST, DISA_TIMR_FIRST_DIGIT_TIMEOUT, 5)
    StartTimer(DISA_TIMR_WAIT_USER_INPUT)
    EnterState(EDISA_STATE_WAIT_DEST_NBR)
end

--====================================
--
-- FSM common state functions
--
--====================================
local function exception()
    SendReleaseCall()
    EndService()
end

local function callDestination()
    TraceInfo("\t callDestination")
    local connArg = CreateConnectArg()
    connArg.destNbr = fsm.disa_dest
    connArg.callerNbrInd = MSS_YES
    connArg.callerNbr = fsm.disa_extension
    connArg.retrieveProfileInd = MSS_YES

    SendConnectWithArg(connArg)
end

local function monitorAnswer()
    TraceInfo("\t monitorAnswer")
    SendRRBE(CALLED_LEGID, DP_O_ANSWER, DP_MON_NOTIFY)
    SendRRBE(CALLED_LEGID, DP_O_DISCONNECT, DP_MON_NOTIFY)
end
--====================================
--
-- FSM state functions
--
--====================================
local function onIdleProc(currMsg)
    TraceInfo("onIdleProc")

    --init inner parameters
    fsm.disa_account = ""
    fsm.disa_dest = ""
    fsm.disa_extension = ""
    fsm.disa_password = ""
    fsm.ctrInd = MSS_NO

    -- check caller id
    local result = db_check_caller_id()
    if result<0 then
        exception()
        return

    elseif result==0 then
        -- the caller is not in the caller_id table
        -- then continue IVR process to input account information
        fsm.ctrInd = MSS_YES
        SendCTR(CALLER_LEGID)
        SendPNC(CALLER_LEGID, DISA_ANNID_INPUT_ACCOUNT, DISA_TIMR_FIRST_DIGIT_TIMEOUT, 5)

        StartTimer(DISA_TIMR_WAIT_USER_INPUT)
        EnterState(EDISA_STATE_WAIT_ACCOUNT)
    else
        -- the calling party pass caller_id authorization
        local disaPfx = ssfGetDisaPrefix()
        TraceInfo("\t\t disa prefix = "..disaPfx)
        local accCodeLen = string.len(disaPfx)
        if string.len(fsm.calledNbr)==accCodeLen then
            askCallerInputDestNbr()
        else
            -- the calling party has inputed destination
            fsm.disa_dest = string.sub(fsm.calledNbr, accCodeLen+1)
            callDestination()
            EndService()
        end

    end

end


local function onWaitAccountProc(currMsg)
    TraceInfo("onWaitAccountProc")
    StopTimer()
    if currMsg==ESCP_EVT_PNC_RESULT then
        fsm.disa_account = currMessage.digits
        TraceInfo("\t user inputs account: "..fsm.disa_account)

        -- check account
        if string.len(fsm.disa_account)==0 then 
            exception() 
            return 
        end

        -- prepare ask password of this account
        askCallerInputPassword()
        return

    else
        exception()
    end
end


local function onWaitPasswordProc(currMsg)
    TraceInfo("onWaitPasswordProc")
    StopTimer()
    if currMsg==ESCP_EVT_PNC_RESULT then
        fsm.disa_password = currMessage.digits
        TraceInfo("\t\t user inputs password: ", fsm.disa_password)
        if string.len(fsm.disa_password)==0 then
            TraceInfo("\t\t wrong password.")
            exception()
            return
        end

        local result = db_check_disa_account_info()
        if result<=0 then
            TraceInfo("\t\t failed to check disa account info.")
            exception()
            return
        end

        askCallerInputDestNbr()
        return
        
    else
        exception()
    end
end


local function onWaitDestinationProc(currMsg)
    TraceInfo("onWaitDestinationProc")
    StopTimer()
    if currMsg==ESCP_EVT_PNC_RESULT then
        fsm.disa_dest = currMessage.digits
        TraceInfo("\t\t user inputs destination number: "..fsm.disa_dest)

        monitorAnswer()
        SendRingBackToneToCaller()
        --Indicate MSS core to route the call to this destination
        callDestination()
        StartTimer(DISA_TIMR_WAIT_ANS)
        EnterState(EDISA_STATE_WAIT_ANS) 
        return

    else
        exception()
    end
end


local function  onWaitAnswerProc(currMsg)
    TraceInfo("onWaitAnswerProc")
    StopTimer()
    if currMsg==ESCP_EVT_ERB and currMessage.edpEvent==DP_O_ANSWER then
        SendDFC(CALLER_LEGID)
    else
        SendReleaseCall()
    end
    EndService()
end

--====================================
--
-- Service main function
--
--====================================
function MainProc(currMsg)
    TraceInfo("DISA: MainProc")    

    if fsm.state==EDISA_STATE_IDLE then
        onIdleProc(currMsg)
    
    elseif fsm.state==EDISA_STATE_WAIT_ACCOUNT then
        onWaitAccountProc(currMsg)

    elseif fsm.state==EDISA_STATE_WAIT_PWD then
        onWaitPasswordProc(currMsg)

    elseif fsm.state==EDISA_STATE_WAIT_DEST_NBR then
        onWaitDestinationProc(currMsg)
    
    elseif fsm.state==EDISA_STATE_WAIT_ANS then
        onWaitAnswerProc(currMsg)

    else   
        exception()
    end
end

function DestroyProc()
    TraceInfo("DestroyProc")   
end