--=======================================================================
-- miniSIPServer pickup service script
--
-- Copyright (C) MYVOIPAPP,Inc. All Rights Reserved.
--
-- author: Gilson
--
-- History:
-- 2014-02-13 Migrated from MSS core.
-- 2016-08-27 Migrated from python script.
--=======================================================================

require "services"
require "utils.data_string"
require "utils.dba"

-- service states
local EPICKUP_IDLE = 0

--====================================
--
-- cache DB functions
--
--====================================
local function cdbGetOBcpID(targetUser)
    TraceInfo("\t cdbGetOBcpID")
    local sqlStr=string.format("select obcpID from tbl_pickup where "..
        "usrNbr='%s' and groupID=%d", targetUser, fsm.groupID)
    local result = cdbQuery(sqlStr)
    if result~=MSS_ERR_OBJ_FOUND then return -1 end

    fsm.obcpID = cdbGetRowIntVal(0)
    return 1
end


local function cdbGetOBcpIDFromGroup()
    TraceInfo("\t cdbGetOBcpIDFromGroup")

    local sqlStr=string.format("select obcpID from tbl_pickup where "..
                "groupID=%d limit 1", fsm.groupID)
    local result=cdbQuery(sqlStr)
    if result~=MSS_ERR_OBJ_FOUND then return -1 end

    fsm.obcpID = cdbGetRowIntVal(0)
    return 1           
end

--====================================
--
-- FSM common functions
--
--====================================
local function initParams()
    fsm.groupID = 0
    fsm.obcpID = 0
end


local function exceptionProc()
    SendReleaseCall()
    EndService()
end


local function procPickUPOne(targetUser)
    TraceInfo("\t procPickUPOne")
    TraceInfo("\t\t target user: "..targetUser)

    local result = cdbGetOBcpID(targetUser)
    if result<0 then
        TraceInfo("\t\t fail to get pickup call information.")
        exceptionProc()
        return
    end

    SendPickUP(fsm.obcpID)
    EndService()
end


local function procPickUPGroup()
    TraceInfo("\t procPickUPGroup")

    local result=cdbGetOBcpIDFromGroup()
    if result<0 then
        TraceInfo("\t\t fail to get pickup call information.")
        exceptionProc()
        return
    end

    SendPickUP(fsm.obcpID)
    EndService()
end


--====================================
--
-- FSM state functions
--
--====================================
local function onIdleProc(currMsg)
    TraceInfo("PickUP: onIdleProc")
    
    fsm.groupID = ssfGetLocalUserPickUPGroupID(fsm.profileID)
    if fsm.groupID==MSS_INVALID_OBJ_ID then
        TraceInfo("\t cannot get pickup group ID")
        exceptionProc()
        return
    end

    local pkOnePrefix = ssfGetPickUpOnePrefix()
    TraceInfo("\t pickup-one prefix is "..pkOnePrefix)
    if StrHasPrefix(fsm.calledNbr, pkOnePrefix)==MSS_TRUE then -- pickup one
        local targetUser=StrLstrip(fsm.calledNbr,pkOnePrefix)
        targetUser=StrRstrip(targetUser,"#")
        procPickUPOne(targetUser)
    else
        procPickUPGroup()
    end
end

--====================================
--
-- Service main function
--
--====================================
function DestroyProc()
    TraceInfo("DestroyProc")
end

function MainProc(currMsg)
    if fsm.state==EPICKUP_IDLE then
        onIdleProc(currMsg)
    else
        TraceInfo("unknown state: "..fsm.state)
        SendReleaseCall()
        EndService()
    end
end